local _, L = ...;
local VERSION = 21200;
local playerName = UnitName("player");
function WorldBuffTracker_GetVersion()
  return VERSION;
end
local wbt_faction = "Alliance";
local wbtString = "[WorldBuffTrackerEvolution] ";
local WorldBuffTrackerFrame = CreateFrame("Frame");
local addonInitialized = 99999999999999;
local WorldBuffTrackerEvents = {}
WorldBuffTracker = {
  ["GuildAnnounce"] = true,
  ["LastSync"] = 0,
  ["HighestVersionSeen"] = VERSION,
  ["1"] = {
    ["Timestamp"] = 0,
    ["Yell"] = 0,
    ["Version"] = VERSION,
    ["Sender"] = playerName,
    ["Origin"] = playerName,
  },
  ["2"] = {
    ["Timestamp"] = 0,
    ["Yell"] = 0,
    ["Version"] = VERSION,
    ["Sender"] = playerName,
    ["Origin"] = playerName,
  },
  ["3"] = {
    ["Timestamp"] = 0,
    ["Yell"] = 0,
    ["Version"] = VERSION,
    ["Sender"] = playerName,
    ["Origin"] = playerName,
  },

}

WorldBuffTrackerZones = {
  ["Horde"] = {
    [1411] = true, [1454] = true, [1413] = true;
  },
  ["Alliance"] = {
    [1453] = true, [1429] = true;
  }
}

WorldBuffTrackerTTB = {
  ["1"] = 13, ["2"] = 17, ["3"] = 17,
}

WorldBuffTrackerBuffCDs = {
  ["1"] = 10800, ["2"] = 21600, ["3"] = 28800,
  ["SF"] = 1500;
}

function WorldBuffTracker_Initialize()
 if WorldBuffTracker["HighestVersionSeen"] == nil then
  WorldBuffTracker["HighestVersionSeen"] = VERSION
end

if(GetSpellBookItemName(1, "spell") ~= nil) then
  if(WorldBuffTrackerSpells[GetLocale()] ~= nil) then
    WorldBuffTrackerSpells = WorldBuffTrackerSpells[GetLocale()];
    WorldBuffTrackerNPCs = WorldBuffTrackerNPCs[GetLocale()];
  else
    WorldBuffTrackerSpells = WorldBuffTrackerSpells["enUS"];
    WorldBuffTrackerNPCs = WorldBuffTrackerNPCs["enUS"];
  end
  wbt_faction, _ = UnitFactionGroup("player");
  if(WorldBuffTracker["LastSync"] - GetServerTime() < -300) then
    C_ChatInfo.SendAddonMessage("WBT-0", VERSION, "GUILD");
    WorldBuffTracker["LastSync"] = GetServerTime();
  end
  addonInitialized = GetTime() + 0.5;
  print('World Buff Tracker Evolution v' .. VERSION .. '  - for more information see /wbt help')
  if(WorldBuffTracker['HighestVersionSeen'] == nil or WorldBuffTracker['HighestVersionSeen'] < VERSION) then
    WorldBuffTracker['HighestVersionSeen'] = VERSION
  end
  if(WorldBuffTracker['HighestVersionSeen'] > VERSION) then
    print("[WorldBuffTracker] " .. L["New update may be available."]);
    return
  end
  WorldBuffTrackerFrame:SetScript("OnUpdate", WorldBuffTracker_DelayTracking);
  WorldBuffTracker_VerifyTable();
  WorldBuffTracker_TimeTableTest();
end
end

function WorldBuffTracker_DelayTracking()
  if(addonInitialized - GetTime() < 0) then
    addonInitialized = 99999999999999;
    WorldBuffTrackerFrame:SetScript("OnUpdate", nil);
    for k, v in pairs(WorldBuffTrackerEvents) do
      WorldBuffTrackerFrame:RegisterEvent(k);
    end
    WorldBuffTrackerFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:UnregisterEvent("CHAT_MSG_MONSTER_YELL");
    WorldBuffTrackerEvents:ZONE_CHANGED_NEW_AREA();
  end
end

function WorldBuffTrackerEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, eventType, _,
  _, sourceName, _, _,
  destGUID, destName, _, _,
  spellId, spellName, _, auraType, _ = CombatLogGetCurrentEventInfo();
  if((eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESHED")
    and destName == playerName and auraType == "BUFF" and WorldBuffTrackerSpells[spellName] ~= nil) then
    local buffID = WorldBuffTrackerSpells[spellName];
    if(buffID == 2) then
      if(WorldBuffTracker["2"]["Yell"] == WorldBuffTracker["3"]["Yell"]) then
        return
      else
        buffID = (WorldBuffTracker["2"]["Yell"] > WorldBuffTracker["3"]["Yell"]) and "2" or "3";
      end
    end
    if(WorldBuffTracker_BuffIsValid(spellName, buffID)) then
      if(IsInGuild()) then
        C_ChatInfo.SendAddonMessage("WBT-" .. buffID, VERSION, "GUILD");
      else
        WorldBuffTracker_HandleBuffById(buffID);
      end
    end
  end
end

function WorldBuffTrackerEvents:ZONE_CHANGED_NEW_AREA(...)
  if(WorldBuffTrackerZones[wbt_faction][C_Map.GetBestMapForUnit("player")] ~= nil) then
    WorldBuffTrackerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:RegisterEvent("CHAT_MSG_MONSTER_YELL");
  else
    WorldBuffTrackerFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:UnregisterEvent("CHAT_MSG_MONSTER_YELL");
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_MONSTER_YELL(msg, npc, ...)
  if(WorldBuffTrackerNPCs[wbt_faction][npc] == nil) then
    return;
  end
  npc = tostring(WorldBuffTrackerNPCs[wbt_faction][npc]);
  WorldBuffTracker_TimeTableTest();
  if(WorldBuffTracker[npc]["Timestamp"] == 0) then
    local t = GetServerTime();
    if(WorldBuffTracker[npc]["Yell"] < t) then
      if(npc ~= "1" or (strfind(msg, L["ThrallYell"]) ~= nil)) then
        C_ChatInfo.SendAddonMessage("WBT-A", npc .. "_" .. VERSION, "GUILD");
        WBTE_wait(math.random(3),WorldBuffTracker_BuffAnnounce,npc, t)
      end
    end
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_ADDON(prefix, message, channel, sender, ...)
  local sender, _ = strsplit("-", sender);
  local t = GetServerTime();
  if(prefix == "WBT-A" and channel == "GUILD") then
    local buff, version = strsplit("_", message);
    version = WorldBuffTracker_GetVersionFromString(version);
    if(WorldBuffTracker[buff] ~= nil and version >= 11326) then
      WorldBuffTracker_BuffAnnounce(buff, t);
    end
  elseif(prefix == "WBT-1") then
    WorldBuffTracker_HandleWarchief(sender, WorldBuffTracker_GetVersionFromString(message), t);
    updateGuildInfo()
  elseif(prefix == "WBT-2") then
    WorldBuffTracker_HandleOnyxia(sender, WorldBuffTracker_GetVersionFromString(message), t);
    updateGuildInfo()
  elseif(prefix == "WBT-3") then
    WorldBuffTracker_HandleNefarian(sender, WorldBuffTracker_GetVersionFromString(message), t);
    updateGuildInfo()
  elseif(prefix == "WBT-0") then
    if(channel == "GUILD" and sender ~= playerName) then
      WorldBuffTracker_SendSyncData(sender)
    elseif(channel == "WHISPER") then
      WorldBuffTracker_HandleSync(message, sender)
      updateGuildInfo()
    end
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_GUILD(prefix, message, channel, sender, ...)
  if string.find(message, '[World Tracker Evolution]', 1, true) ~= nil then 
    guildTimer = GetServerTime()
  end
end

function WorldBuffTracker_BuffAnnounce(buffID, t)
  if(WorldBuffTracker[buffID]["Yell"] < t) then
    WorldBuffTracker[buffID]["Yell"] = t + 3600;
    local buffString;
    for k,v in pairs(WorldBuffTrackerSpells) do
      if(tostring(v) == buffID or (v == 2 and buffID == "3")) then
        buffString = k;
        break;
      end
    end
    local buffInfo = "";
    if(buffID ~= "1") then
      buffInfo = buffID == "2" and " (Onyxia) " or " (Nefarian) ";
    end
    if(t - guildTimer > 30 and WorldBuffTracker['GuildAnnounce']) then
      SendChatMessage(wbtString .. buffString .. buffInfo .. " incoming!", "GUILD")
    end
  end
end

local function WorldBuffTrackerFrame_Command(msg, editbox)
 if(strlower(msg) == "announce") then
  if(WorldBuffTracker["GuildAnnounce"]) then
   print(wbtString .. L["Guild announce: disabled."])
 else
   print(wbtString .. L["Guild announce: enabled."])
 end
 WorldBuffTracker["GuildAnnounce"] = not WorldBuffTracker["GuildAnnounce"];
 return
elseif(strlower(msg) == "version") then
  print("World Buff Tracker Evolution version:" .. VERSION)
  return
elseif(strlower(msg) == "guildinfo") then
  if(WorldBuffTracker["GuildInfo"]) then
   print(wbtString .. L["Guild Info: disabled."])
 else
   print(wbtString .. L["Guild Info: enabled."])
 end
 WorldBuffTracker["GuildInfo"] = not WorldBuffTracker["GuildInfo"]   
 return     
elseif(strlower(msg) == "help") then	
  print('--- World Buff Tracker Evolution ---')
  print('Available commands:')
  print('/wbt')
  print('/wbt version')
  print('/wbt announce')
  print('/wbt guildinfo')
  print('/wbt help')
  return

elseif(strlower(msg) == "sync") then	
  C_ChatInfo.SendAddonMessage("WBT-0", VERSION, "GUILD")
  updateGuildInfo()	
  return
elseif(strlower(msg) == "") then
    WorldBuffTracker_TimeTableTest();
  local t = GetServerTime();
  local text = "   --- World Buff Tracker Evolution ---" .. "\n"  
  local buffs = {};
  for k, v in pairs(WorldBuffTrackerSpells) do
    buffs[v] = k;
  end
  for v=1,3 do
    local k = buffs[((v > 2) and 2 or v)];
    local buffSource = "";
    if(v == 1) then
      text = text .. "Warchief's Blessing:\n";
    elseif(v == 2) then
      text = text .. "Dragonslayer (Onyxia):\n";
    elseif(v == 3) then 
      text = text .. "Dragonslayer (Nefarian):\n";
    end
    
    v = tostring(v);
    local bT = WorldBuffTracker[v]["Timestamp"];
    if(bT > 0) then
      local cd = WorldBuffTrackerBuffCDs[v];
      local sources = " ";      
      text = text .. "    " .. string.format(L["Popped: %s  --    Next: %s\n"], date('%H:%M:%S', bT), date('%H:%M:%S', bT + cd))
    else
      text = text .. ("    " .. L["No data."]) .. "\n"
    end
    
  end
  text = text .. "-----------------"

  print(text)
end
end


wbte_string = "%-%-%-%sWorld%sBuff%sTracker%sEvolution%s%-%-%-.+%(%d+#%d+#%d+%)"
function updateGuildInfo()
	if(not CanEditGuildInfo() or not WorldBuffTracker["GuildInfo"]) then
		return
	end

  if(WorldBuffTracker['HighestVersionSeen'] > VERSION) then
    return
  end


  local oldtext = GetGuildInfoText()
  local server_time = GetServerTime()

  if (string.len(oldtext) <= 0) then
    return
  end

  local _, _, warchief_guild_timer, onyxia_guild_timer,  nefarian_guild_timer= string.find(oldtext, '%((%d+)#(%d+)#(%d+)%)') 

  if (string.find(oldtext, '%-%-%-%sWorld%sBuff%sTracker%sEvolution%s%-%-%-') == nil) then  		
    SetGuildInfoText(oldtext .. "\n" .. GuildTimeTable())
    return
  elseif (WorldBuffTracker["1"]["Timestamp"] - warchief_guild_timer > 30 ) then
    local newtext = string.gsub(oldtext, wbte_string, GuildTimeTable())       
    SetGuildInfoText(newtext)
    return        	
  elseif (WorldBuffTracker["2"]["Timestamp"] - nefarian_guild_timer > 30) then
    local newtext = string.gsub(oldtext, wbte_string, GuildTimeTable())       
    SetGuildInfoText(newtext)     
    return
  elseif (WorldBuffTracker["2"]["Timestamp"] - onyxia_guild_timer > 30) then
    local newtext = string.gsub(oldtext, wbte_string, GuildTimeTable())       
    SetGuildInfoText(newtext)     
    return
  elseif(warchief_guild_timer - server_time < 0 or nefarian_guild_timer - server_time < 0 or onyxia_guild_timer - server_time < 0) then
    local newtext = string.gsub(oldtext, wbte_string, GuildTimeTable())       
    SetGuildInfoText(newtext)     
    return
  end

end

function GuildTimeTable( ... )
  WorldBuffTracker_TimeTableTest();
  local t = GetServerTime();
  local text = "--- World Buff Tracker Evolution ---" .. "\n\n"  
  local buffs = {};
  for k, v in pairs(WorldBuffTrackerSpells) do
    buffs[v] = k;
  end
  for v=1,3 do
    local k = buffs[((v > 2) and 2 or v)];
    local buffSource = "";
    if(v == 1) then
      text = text .. "Warchief's Blessing:\n";
    elseif(v == 2) then
      text = text .. "Dragonslayer (Onyxia):\n";
    elseif(v == 3) then 
      text = text .. "Dragonslayer (Nefarian):\n";
    end
    
    v = tostring(v);
    local bT = WorldBuffTracker[v]["Timestamp"];
    if(bT > 0) then
      local cd = WorldBuffTrackerBuffCDs[v];
      local sources = " ";      
      text = text .. "    " .. string.format(L["Popped: %s \n    Next: %s \n\n"], date('%H:%M:%S', bT), date('%H:%M:%S', bT + cd))
    else
      text = text .. ("    " .. L["No data."]) .. "\n\n"
    end
  end

  text = text .. "--- Last update: " .. UnitName("player") .. " - " .. date('%H:%M:%S', GetServerTime()) .. " ---"
  text = text .. "(" .. WorldBuffTracker["1"]['Timestamp'] .. "#" .. WorldBuffTracker["2"]['Timestamp'] .. "#" .. WorldBuffTracker["3"]['Timestamp'] .. ")"
  return text;
end
function WorldBuffTracker_TimeUntilDate(seconds)
  local str = "";
  if(seconds > 3600) then
    local hours = math.floor(seconds / 3600);
    str = hours .. "h ";
    seconds = seconds - 3600 * hours;
  end
  if(seconds > 60) then
    local minutes = math.floor(seconds / 60);
    str = str .. minutes .. "m ";
    seconds = seconds - 60 * minutes;
  end
  if(seconds > 0) then
    str = str .. seconds .. "s";
  end
  return str;
end

function WorldBuffTracker_AlertPlayer(buff, playSound, whisper)
  if(playSound) then
    PlaySoundFile("Interface/AddOns/WorldBuffTracker/resources/sound/beware.ogg", "Master");
  end
  if(whisper) then
    SendChatMessage(wbtString .. buff .. " incoming!", "WHISPER", nil, UnitName("player"));
  end
end

SLASH_WORLDBUFFS1, SLASH_WORLDBUFFS2, SLASH_WORLDBUFFS3 = '/wbt'

SlashCmdList["WORLDBUFFS"] = WorldBuffTrackerFrame_Command;

WorldBuffTrackerFrame:SetScript("OnUpdate", WorldBuffTracker_Initialize);
WorldBuffTrackerFrame:SetScript("OnEvent", function(self, event, ...)
  WorldBuffTrackerEvents[event](self, ...);
end);
C_ChatInfo.RegisterAddonMessagePrefix("WBT-0");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-1");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-2");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-3");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-A");
